import sys, os
import getpass
import subprocess
import urllib
import requests
import configparser
from pathlib import Path
import json

from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

test_dir = Path(__file__).resolve().parent

config = configparser.ConfigParser()
config.read(test_dir / 'config.ini')

url = config['Default']['uploadURL']
token = config['Default']['token'].strip()

if len(token) == 0:
    print("config.iniにtokenを設定してください")
    exit(1)

unit = config['Default']['unit']

def info(s):
    print(f"\033[32m{s}\033[0m")

def warning(s):
    print(f"\033[31m{s}\033[0m")

def compile(question):
    try:
        Path('bin').mkdir(exist_ok=True)
        subprocess.check_call(f"gcc {question}.c -o bin/{question}".split(" "))
    except:
        warning("Can't compile your file")
        exit(1)

def assert_pe(q, a, case_name, args = None):
    q = q.strip()
    a = a.strip()

    if (q == a):
        info(f"{case_name}...OK")
        return True
    else:
        warning(f"{case_name}...Failed")
        if args is not None:
            warning(f"test case is {args}")
        return False

def do_test(question, testcase) -> bool:
    try:
        seq = [assert_pe(subprocess.check_output([f"bin/{question}"] + d['args'], timeout=5).decode(), d['output'], d['name'], d['args']) for d in testcase]

        if all(seq):
            info("All tests succeeded")
            return True
        else:
            warning("Test Failed")

    except subprocess.TimeoutExpired:
        warning('Test Failed: Time Out')

    except Exception as e:
        print(e)
        warning('Test Failed: Get the bugs out of your program!')

    return False

def get_testcase(url, token, unit, question):
    data = json.dumps({'token' : token, 'unit': int(unit), 'question': int(question)}).encode("utf-8")
    response = requests.post(url + "testcase", data=data, headers={"Content-Type": "application/json"}, verify=False)
    if response.status_code == 401:
        warning('your token is invalid')
        exit(1)

    return response.json()['case']

def post_code(url, token, unit, question):
    with open(f"{question}.c", 'r') as f:
        code = f.read()
    data = json.dumps({'token' : token, 'unit': int(unit), 'question': int(question), 'code': code}).encode("utf-8")
    response = requests.post(url + "postcode", data=data, headers={"Content-Type": "application/json"}, verify=False)
    return response


question = sys.argv[1]

if question == "status":
    data = json.dumps({'token' : token}).encode("utf-8")
    response = requests.post(url + "status", data=data, headers={"Content-Type": "application/json"}, verify=False)
    print(response.json()['status'])
    exit(0)


compile(question)
testcase = get_testcase(url, token, unit, question)
is_ok = do_test(question, testcase)

if is_ok:
    status = post_code(url, token, unit, question)
